function [p_out, v_out, q_out, xi_out, b_g_out, b_a_out, sigma, dot_b_hat, omega_hat] = ....
    observer_thb( T_s, status, f_IMU, omega_IMU, mag_IMU,.... 
    p_n_GNSS, gnss_var_w, v_n_GNSS, GNSS_sampled, f_GNSS, m_n, gnss_z_bias,.... 
    xy_model, uv_model , var_GNSS)

global time
% OBSERVER The Grip PVA observer 2013. Computes PVA estimates based on INS
% ans GNSS measurments
%
% Input: 
% T_s: Observer sampling time 
% stauts: Flag deciding the attitude observer mode
% time: not used in current version, but nice for debugging purposes using 
% conditional brake point
% f_IMU: Specific force measured by the IMU
% omega_IMU: Angular velocity measured by the IMU
% mag_IMU: Magnetic field measured by the IMU
% p_n_GNSS: Position measurement
% v_n_GNSS: Velocity measurement
% GNSS_available: binary value. 1 if new GNSS measurement is available
% f_GNSS: GNSS sampling frequency
% m_n: magnetic field of current location on earth
%
% Output:
% estimates of position, velocity, attitude, xi, gyro bias and acc bias
%
% Email: torleiv.haland.bryne@gmail.com

% Integrator step length
h = T_s;

% NED gravity constant
g_n = [0 0 9.81]';

I_3 = eye(3); I_4 = eye(4); mat_0 = zeros(3,3); vec_0 = [0 0 0]';

% Projection limits gyro bias estimates 
Mb = 0.5*pi/180; Mb_hat = 0.51*pi/180;

% Projection limits acc bias estimates 
m_f = 0.9*norm(g_n); M_f = 1.1*norm(g_n);
delta = 0.9*m_f;
M_w = 1; M_w_hat = 1.1*M_w;

persistent p_pred v_pred q_pred xi_pred b_g_hat w_hat eig_val Gamma p_fix v_fix

if isempty( p_pred ) || isempty( v_pred ) || isempty( q_pred ) ....
        || isempty( xi_pred ) || isempty( b_g_hat ) || isempty( w_hat )
    p_pred = p_n_GNSS; v_pred = [0 0 0]'; q_pred = euler2q(0, 0, 0);
    xi_pred = [0 0 0]'; b_g_hat = [0 0 0]'; w_hat = [0 0 0 0]';
end
if isempty( p_fix ) || isempty( v_fix )
    p_fix = [0 0 0]';
    v_fix = [0 0 0]';
end

%% Observer Gains for PV estimates

if GNSS_sampled == 1    
    K_pv = I_3*2.737;
    K_vv = I_3*2.363;
    K_xiv = I_3*1.068;
    
%     K_pp = I_3*0.6*(1+2*exp(-0.*time));
    K_pp_0 = 0.3*I_3;
    beta = 0.75;
    alpha = 2;
    K_var = beta*diag( [exp( -alpha*gnss_var_w(1) ) exp( -alpha*gnss_var_w(2)) exp( -alpha*gnss_var_w(3) ) ] );
    K_pp = K_pp_0 + K_var;
    
    % initalzation increase
    alpha_pv = 0.1;    
    beta_pv = 2;
    K_pp = K_pp*(1 + beta_pv*exp(-alpha_pv*time) );
    K_vp = I_3*0.11*( 1 + beta_pv*exp(-alpha_pv*time) );
    K_xip = I_3*0.006;
    
    K_bias = vec_0;
    theta = 2;
elseif GNSS_sampled == 0
    K_pv = mat_0;
    K_vv = mat_0;
    K_xiv = mat_0;

    K_pp = mat_0;
    K_vp = mat_0;
    K_xip = mat_0;
    K_bias = vec_0;
    theta = 0;
end
C_v = I_3*0; 

switch status
    % dead-reckogning
    case -1
        k_1 = 1;
        k_2 = 0;
        k_I = 0;
        Gamma  = zeros(4,4);
        theta = 0;
    case 0 % init phase. fast gyro bias estimation  
        k_1 = 2*10;
        k_2 = 1*10;
        k_I = .01;
        Gamma  = zeros(4,4);
    case {1, 2, 3, 4, 5} % nominal. no acceleromter bias
%         k_1 = 2*5;
%         k_2 = 1.5*5;
%         k_1 = 2;
%         k_2 = 1.5;
%         k_I = .009;
        k_1 = 1*(1 + 3*exp( -0.2*time ));
        k_2 = 0.55*(1 + 3*exp( -0.2*time ));
        k_I = .008;
        
        Gamma  = zeros(4,4);
%         xi_pred = [0 0 0]';
%         v_pred = [0 0 0]';
    case 6 % voting has retured discrepancy between sensors
        Gamma  = zeros(4,4);
        k_1 = 1;
        k_2 = 0.55;
        k_I = 0.008;
        K_pp = 0.8*K_pp;
        K_vp = 0.8*K_vp;
    case 8 % acc bias estimation 
        k_1 = 1;
        k_2 = 1.5;
        k_I = .008;
        gamma_z = 0.25;
%         gamma_x = 50*gamma_z;
%         gamma_y = 50*gamma_z;
        gamma_x = 225*gamma_z;
        gamma_y = 225*gamma_z;  
        Gamma  = diag([1 gamma_x gamma_y gamma_z])*1e-4;
    otherwise
        disp(status)
        error('Unknown status');
end
% checking the eigenvaules of the observer
if isempty(eig_val) && GNSS_sampled == 1
    A = [mat_0 I_3 mat_0
         mat_0 mat_0 I_3
         mat_0 mat_0 mat_0];
    C = [I_3 mat_0 mat_0
        mat_0 C_v mat_0];
    K = [K_pp K_pv;
        K_vp K_vv;
        K_xip K_xiv];
    eig_val = eig( A - K*C );
end


% Estimate of the quaternion rotation matrix
R_hat = Rquat(q_pred);

b_a_hat = w_hat(2:4);
f_b = f_IMU - b_a_hat;
% f_b_bar = f_b/norm( f_b );
f_b_bar = f_b / max( [norm( f_b ), delta] );

f_n_hat = R_hat*f_b + xi_pred;
f_n_hat_bar = f_n_hat / max( [norm( f_n_hat ), delta]);
% f_n_hat_bar = f_n_hat / norm( f_n_hat );

m_b_bar = mag_IMU / norm( mag_IMU );

m_b_bar = cross(m_b_bar, f_b_bar);

m_n_bar = m_n / norm( m_n );
m_n_bar = cross(m_n_bar, f_n_hat_bar);

% Predictions
m_b_bar_hat = R_hat'*m_n_bar;
f_b_bar_hat = R_hat'*f_n_hat_bar;

% Calculate the nonlinear attitude indjection term
sigma = k_1*cross(m_b_bar, m_b_bar_hat) + k_2*cross(f_b_bar, f_b_bar_hat);

omega_hat = omega_IMU - b_g_hat + sigma;
% omega_hat =  sigma;
% w_hat(1) = norm(b_a_hat);
w_hat_norm = norm(w_hat);
% y_hat = norm(f_n_hat)^2 - norm(f_IMU)^2;
y_hat = max( norm(f_n_hat)^2, M_f^2 ) - norm(f_IMU)^2;

% Regressor
phi = [1; -2*f_IMU];

%% Observer Equations And Integration
p_n_pos_ref = p_n_GNSS;
v_n_vel_ref = v_n_GNSS; 
if status == 4
    p_n_pos_ref = [xy_model' 0]';
end
% v_hat = v_pred + h*f_GNSS*( theta^2*K_vp*(p_n_GNSS - p_pred - K_bias*gnss_z_bias) + theta*K_vv*(C_v*p_n_pos_ref - C_v*v_pred) );

v_hat = v_pred + h*f_GNSS*( theta^2*K_vp*(p_n_pos_ref - p_pred - K_bias*gnss_z_bias) + theta*K_vv*(C_v*v_n_vel_ref - C_v*v_pred) );
% if status == 4
%     p_fix = [xy_model' p_n_GNSS(3) - gnss_z_bias]';
%     v_fix = [uv_model' 0]';
%     v_hat = v_pred + h*f_GNSS*( theta^2*K_vp*(p_fix - p_pred) + theta*K_vv*(v_fix - v_pred) );
% end
v_pred_next = v_hat + h*(f_n_hat + g_n);

p_hat = p_pred + h*f_GNSS*( theta*K_pp*(p_n_pos_ref - p_pred - K_bias*gnss_z_bias) + K_pv*(C_v*v_n_vel_ref - C_v*v_pred) );
% if status == 4
%     p_hat = p_pred + h*f_GNSS*( theta*K_pp*(p_fix - p_pred) + K_pv*(v_fix - v_pred) );
% end  
p_pred_next = p_hat + h*v_pred_next;

% omega = 0.8;
% zeta = 0.1;
% den = [1 2*omega omega^2];
% num = [1 2*omega*zeta omega^2];
% p_pred_next = filter(num, den, p_pred_next); 

q_hat = q_pred + h*T_q(q_pred)*omega_hat;
q_pred_next = q_hat;
q_pred_next = q_pred_next/norm(q_pred_next);

% Gyro Bias Estimation
dot_b_hat = -k_I*sigma;
b_g_hat_norm_2 = norm(b_g_hat)^2;
if b_g_hat_norm_2 > Mb^2 && b_g_hat'*dot_b_hat > 0
    c = min([1, (b_g_hat_norm_2 - Mb^2)/(Mb_hat^2 - Mb^2)]);
%     c = 1;
    dot_b_hat = (I_3 - c*(b_g_hat*b_g_hat')/b_g_hat_norm_2)*dot_b_hat;
end
        
% Accelerometer Bias Estimation
w_hat_dot = Gamma*phi*(y_hat - phi'*w_hat); 
if w_hat_norm^2 > M_w_hat^2 && w_hat'*w_hat_dot > 0
%     c = min([1, (w_hat_norm^2 - M_w^2)/(M_w_hat^2 - M_w^2)]);
    c = 1;
    w_hat_dot = (I_4 - c*Gamma*(w_hat*w_hat')/(w_hat'*Gamma*w_hat))*w_hat_dot;
end

xi_hat = xi_pred + h*f_GNSS*(theta^3*K_xip*(p_n_pos_ref - p_pred - K_bias*gnss_z_bias) + theta^2*K_xiv*(C_v*v_n_vel_ref - C_v*v_pred));
xi_pred_next = xi_hat - h*R_hat*Smtrx(sigma)*f_b + h*R_hat*w_hat_dot(2:4);

b_g_hat = b_g_hat + h*dot_b_hat;    
w_hat = w_hat + h*w_hat_dot;
w_hat(1) = norm(w_hat(2:3));

% Updates internal states before next interation
p_pred = p_pred_next;
v_pred = v_pred_next;
q_pred = q_pred_next;
xi_pred = xi_pred_next;

% Output
p_out = p_pred_next;
v_out = v_pred_next;
q_out = q_pred_next;
xi_out = xi_pred_next;
b_g_out = b_g_hat;
b_a_hat = w_hat(2:4);
b_a_out = b_a_hat;

end

