function [tau_wave_out, wave_model_params] = wave_generator( dt, N, sigma_wave, vessel_type, mass_matrix, save_flag )
    
    if (exist('wave_forces_and_moments.mat','file') == 0 || save_flag == 1)
        tic;
        tau_wave_out = zeros( N+1, 6 );
        
        w_0 = 0.5;
        H_s = 7;
        w_max = 3;

        w = (0.0001:0.01:w_max)';  
        T_0 = 2*pi/w_0;
        s_T_0= sprintf('%0.3g', T_0);
        s_output = strcat('Wave periode: ', s_T_0);
        disp(s_output);
        
        % JONSWAP
        gamma = 3.3;
        S = wavespec_mod(7,[H_s, w_0, gamma], w, 1);
%         S = wavespec(7, [H_s, w_0, gamma], w,1);
        sigma = sqrt(max(S));

        lambda = lsqcurvefit('PSD_linear', 0.1, w, w_0, sigma, S);
        
        wave_model_params = struct( 'omega', w_0, 'lambda', lambda);
        
        P_yy = PSD_linear(lambda,w, w_0, sigma);
        hold on; 
        plot(w,P_yy,'linewidth',2); 
        hold off; 
        s_legend= {'JONSWAP spectrum','Linear approximation'};
        legend(s_legend, 'Location', 'best', 'Interpreter','Latex');
        grid on;
        set(gca,'FontSize',12)
        resize_fig('wave_spec_plot');
        s_fig_file_name = 'JONSWAP_spectrum_wave_spec';
        s_fig_path_and_name = strcat(pwd, '\figs\', s_fig_file_name);
        export_fig(s_fig_path_and_name, '-pdf','-transparent');
        
        % Linear Model for First-order Wave-Inducesed Forces and Moments
        zero_mat = zeros( 6, 6 ); I_6 = eye( 6,6 ); 
        k_w = 2*lambda*w_0*sigma;
        
        Omega_0 = I_6*w_0; Lambda = lambda*I_6; K_w = I_6*k_w;
        A_w = [ zero_mat            I_6;
              -Omega_0^2  -2*Lambda*Omega_0];
        E_w = [zero_mat K_w]';
        C_w = [zero_mat I_6];
        eigenvalues = eig(A_w - E_w*C_w);
        disp(eigenvalues)
        
        mean = 0;
        w_1 = normrnd(mean, sigma_wave(1), N+1, 1);
        w_2 = normrnd(mean, sigma_wave(2), N+1, 1);
        w_3 = normrnd(mean, sigma_wave(3), N+1, 1);
        w_4 = normrnd(mean, sigma_wave(4), N+1, 1);
        w_5 = normrnd(mean, sigma_wave(5), N+1, 1);
        w_6 = normrnd(mean, sigma_wave(6), N+1, 1);
        w = [w_1 w_2 w_3 w_4 w_5 w_6];
        x_0 = zeros( 6*2, 1);
%         x = x_0;
        % Main loop
        h = dt;
        for k = 1:N+1
            u = w(k,:)';
            % RK 4 Integration
            k_1 = h*( A_w*x_0 + E_w*u );
            x  = x_0 + 0.5*k_1;
            k_2 = h*( A_w*x + E_w*u );
            x  = x_0 + 0.5*k_2;
            k_3 = h*( A_w*x + E_w*u );
            x  = x_0 + k_3;
            k_4 = h*( A_w*x + E_w*u );
            x = x_0 + (k_1 + 2*(k_2 + k_3) + k_4)/6;
            
            tau_wave_out(k,:) = mass_matrix*C_w*x;
        end
        % saving wave forces to file
        save_file = 'wave_forces_and_moments';
        save(save_file, 'tau_wave_out', 'wave_model_params');
        
        t_stop = toc;
        s_output = strcat('Time to compute wave forces and moments: ', sprintf('%g', t_stop));
        disp(s_output);
    else
       load('wave_forces_and_moments'); 
    end
end

function P_yy = PSD_linear( lambda, w, w_0, sigma)
% Pyy = PSD_linear(lambda,w) 2nd-order linear power spectracl density (PSD)
% function
%
%   w       = wave spectrum frequency (rad/s)
%   lambda  = relative damping factor
%
P_yy = 4*( lambda*w_0*sigma )^2*w.^2 ./ ( (w_0^2 - w.^2).^2 + 4*(lambda*w_0.*w).^2 );

end