function S = wavespec_mod(SpecType,Par,W,PlotFlag)
%Function to evaluate different type of wave spectra.
%
%Use:  S = wavespec(SpecType,Par,W,PlotFlag)
%
%Input:
% SpecType	- Spectrum type
% Par		- Spectrum parameters
% W			- Column vector of wave frequencies [rad/s]
% PlotFlag	- 1 to plot the spectrum, 0 for no plot
%
%Output:
% S			- Column vector of wave spectrum values [m^2 s]; evaluated at W(k) 
%     
%SpecType and Par =[p1,p2,p3,...pn]:
% SpecType =7, JONSWAP (p1=Hs,p2=w0,p3=gamma)
%
%JONSWAP (Hs,w0, gamma):
% p1 = Hs - Significant wave height (Hs = 4 sqrt(m0)) [m]
% p2 = w0 - Modal Freq. [rad/sec] (Recomended 1.25<w0*sqrt(Hc)<1.75)
% p3 = gamma - Peakedness factor (Recommended between 1 and 5; usually 3.3, set to zero to use DNV formula)
% alpha=0.2*Hc^2*w0^4/g^2;
% g=9.81 [m/s^2]
% sigma=0.07  if w<w0, sigma=0.09 otherwise;
% S(w)=S1*S2  [m^2 s]  
%with,
% S1=alpha*g^2*(W^-5)*exp(-(5/4)*(w0/w)^4);
% S2=gamma^(exp(-(w-w0)^2/(2*(sigma*w0)^2))); 
% Reference [3]
%
% References: 
%
% [1] A.R.M.J LLoyd (1998) "Seakeeping: Ship Behaviour in Rough Wheather."
%     Published by A.R.M.J LLoyd, Gosport UK.ISBN 0-9532634-01
%
% [2] Ochi, M.K. (1998) "Ocean Waves, The stochastic Approach"
%     Cambridge Ocean Technology Series, Vol 6,
%     Cambridge University Press.
%
% [3] Srensen, A.J. (2005) "Marine Cybernetics: Modelling and Control"
%     Lecture Notes for TMR4241 Marine Control Systems, NTNU, 2005.
%
% [4] K.Torsethaugen (1996): "Model for a Doubly Peaked Wave Spectra"
%      Sintef report no.: STF22 A96204 prepared for Norsk Hydro.
%
% [5] T.I. Fossen (2002) "Marine Control Systems" Marine Cybernetics. 
%
% [6] Lewis E.V. "Principles of Naval Architecture volume  III
%     Motions in Waves and Controllability." SNAME, 1989.
%
% Created by: Tristan Perez in 2005  
% Revisions: 2007-03-09 minor fixes yvind Smogeli
%             Adapted for MSS V3.0 March 2005. Grouped all spectra in one
%             function. Added Spec_type=1,4,5,6.
%            2009-09-11 fixed scaling of JONSWAP sepctrum
% ________________________________________________________________
%
% MSS GNC is a Matlab toolbox for guidance, navigation and control.
% The toolbox is part of the Marine Systems Simulator (MSS).
%
% Copyright (C) 2008 Thor I. Fossen and Tristan Perez
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% E-mail: contact@marinecontrol.org
% URL:    <http://www.marinecontrol.org>

S=[];
[m,n] = size(W);
if n>m
   disp('Error: W must be a column vector')
   return
end

switch SpecType,
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
case 7,%JONSWAP (gamma,Hs,w0):
    Hs	  = Par(1);
    w0    = Par(2);
    gamma = Par(3);
    g=9.81;
    alpha=0.2*Hs^2*w0^4/g^2;
    if gamma<1 | gamma > 7,  %DNV recommeded values when gamma is unknown.
		if gamma ~= 0
			% Display warning if gamma is outside validity range and not
			% set to zero
			disp('Warning: gamma value in wave_spectrum function outside validity range, using DNV formula')
		end
		k=2*pi/(w0*sqrt(Hs));
       if k <= 3.6
           gamma = 5;
       elseif k <= 5
		   gamma = exp(5.75-1.15*k);
	   else % k > 5
		   gamma = 1;
	   end
    end
    for k=1:1:length(W),
            if  W(k) < w0,
                 sigma=0.07;
            else
                 sigma=0.09;
            end
            S1=alpha*g^2*(W(k)^-5)*exp(-(5/4)*(w0/W(k))^4);
            S2=gamma^(exp(-(W(k)-w0)^2/(2*(sigma*w0)^2)));
            % DNV conversion factor from <Environmenatal conditions and environmental
            % loads. April 2007, DNV-RP-C205>            
            Conv_factor =  1-0.287*log(gamma); 
            S=[S;Conv_factor*S1*S2];
    end
    TitleStr='JONSWAP Spectrum';
    L1Str = ['gamma =',num2str(gamma),' Hs =',num2str(Hs),' [m],','  w0=',num2str(w0),' [rad/s]'];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
otherwise
    disp('Wrong spectrum type identifier, SpecType=1,2,..,8')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Plot Spectrum
if PlotFlag==1,
	plot(W, S,'r','linewidth',1)
	title(TitleStr)
    legend(L1Str)
	xlabel('$\omega$ [rad/s]')
	ylabel('$S(\omega)$ [m$^2$ s]')
    grid on;
end            
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% End of Function wave_spectrum

